/*
Copyright © CINES - Centre Informatique National pour l'Enseignement Supérieur (2020)

[dad@cines.fr]

This software is a computer program whose purpose is to provide
a web application to create, edit, import and export archive
profiles based on the french SEDA standard
(https://redirect.francearchives.fr/seda/).


This software is governed by the CeCILL-C  license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/ or redistribute the software under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.
*/

package fr.gouv.vitamui.pastis.controller;

import com.fasterxml.jackson.core.JsonProcessingException;
import fr.gouv.vitamui.pastis.model.ElementProperties;
import fr.gouv.vitamui.pastis.model.profiles.PastisProfile;
import fr.gouv.vitamui.pastis.model.profiles.ProfileNotice;
import fr.gouv.vitamui.pastis.model.profiles.ProfileResponse;
import fr.gouv.vitamui.pastis.util.*;
import io.swagger.v3.oas.annotations.OpenAPIDefinition;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.info.Info;
import io.swagger.v3.oas.annotations.servers.Server;
import io.swagger.v3.oas.annotations.servers.ServerVariable;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import java.io.*;
import java.net.URISyntaxException;
import java.util.List;


@OpenAPIDefinition(tags = { @Tag(name = "pastis") },
    info = @Info(title = "Pastis Rest Api", description = ""),
    servers = { @Server(url = "dev.vitamui.com" ,
        variables = { @ServerVariable(name = "scheme", allowableValues = {"https", "http"}, defaultValue = "https"),
            @ServerVariable(name = "port", description = "Api port",defaultValue = "8051")})
        , @Server(url = "localhost" ,
        variables = { @ServerVariable(name = "scheme", allowableValues ={"https", "http"}, defaultValue = "http"),
            @ServerVariable(name = "port", description = "Api port", defaultValue = "8051")}) })
@RestController
class ProfileController {

    private static final Logger LOGGER = LoggerFactory.getLogger(ProfileController.class);

    private static final String APPLICATION_JSON_UTF8 = "application/json; charset=utf-8";

    @Autowired
    private ProfileService profileService;

    @Operation(summary = "method for test Api is running",
        description = "method for test Api is running",
        tags = {"pastis"})
    @RequestMapping(value = "/test", method = RequestMethod.GET)
    ResponseEntity<String> test() {
        if (profileService.test() != null) {
            return new ResponseEntity<>(profileService.test(), HttpStatus.OK);
        }	else {
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @Operation(summary = "Retrieve RNG representation of the JSON structure",
        description = "Retrieve RNG representation of the JSON structure of archive profile",
        tags = {"pastis"})
    @RequestMapping(value = "/getarchiveprofile", method = RequestMethod.POST, consumes = APPLICATION_JSON_UTF8, produces = MediaType.APPLICATION_XML_VALUE)
    ResponseEntity<String> getArchiveProfile(@RequestBody final ElementProperties json) throws IOException  {
        if (profileService.getArchiveProfile(json) != null) {
            return ResponseEntity.ok(profileService.getArchiveProfile(json));
        }	else {
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @Operation(summary = "Retrieve JSON representation of archive unit profile",
        description = "Retrieve JSON representation of archive unit profile",
        tags = {"pastis"})
    @RequestMapping(value = "/getarchiveunitprofile", method = RequestMethod.POST, consumes = APPLICATION_JSON_UTF8, produces = MediaType.APPLICATION_JSON_VALUE)
    ResponseEntity<String> getArchiveUnitProfile(@RequestBody final ProfileNotice json) throws IOException  {
        if (profileService.getArchiveUnitProfile(json) != null) {
            return ResponseEntity.ok(profileService.getArchiveUnitProfile(json));
        }	else {
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }

    }

    @RequestMapping (value = "/getfile", method = RequestMethod.GET, produces = "text/plain")
    ResponseEntity<String> getFile() {
        if (profileService.getFile() != null) {
            return new ResponseEntity<>(profileService.getFile(), HttpStatus.OK);
        }	else {
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @Operation(summary = "Retrieve JSON representation of the RNG structure",
        description = "Retrieve JSON representation of the RNG structure",
        tags = {"pastis"})
    @RequestMapping (value = "/createprofile", method = RequestMethod.GET)
    ResponseEntity<ElementProperties> createProfile() throws URISyntaxException {

        if (profileService.createProfile() != null) {
            return ResponseEntity.ok(profileService.createProfile());
        }	else {
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @Operation(summary = "Load Profile",
        description = "Load profile by Id",
        tags = {"pastis"})
    @RequestMapping (value = "/edit", method = RequestMethod.POST)
    ResponseEntity<ProfileResponse> loadProfile(@RequestParam(name = "id") String id) throws JsonProcessingException {
        if (profileService.loadProfile(id) != null) {
            return ResponseEntity.ok(profileService.loadProfile(id));
        }	else {
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @Operation(summary = "Upload profile PA or PUA",
        description = "Upload profile PA or PUA",
        tags = {"pastis"})
    @RequestMapping (value = "/profile", method = RequestMethod.POST,
            consumes = "multipart/form-data", produces = "application/json")
    ResponseEntity<ProfileResponse> loadProfileFromFile(@RequestParam MultipartFile file) {
        if (profileService.loadProfileFromFile(file) != null) {
            return ResponseEntity.ok(profileService.loadProfileFromFile(file));
        }	else {
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @Operation(summary = "Generate Json Struct from RNG file",
        description = "Generate Json Struct from RNG file",
        tags = {"pastis"})
    @RequestMapping (value = "/createprofilefromfile",
            method = RequestMethod.POST,consumes = "multipart/form-data",
            produces = "application/json")
    ResponseEntity<ElementProperties> createProfileFromFile(@RequestParam MultipartFile file )  {
        if (profileService.createProfileFromFile(file) != null) {
            return ResponseEntity.ok(profileService.createProfileFromFile(file));
        }	else {
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @Operation(summary = "Retrieve all profiles PA and PUA",
        description = "Retrieve all profiles PA and PUA",
        tags = {"pastis"})
    @RequestMapping (value = "/getprofiles", method = RequestMethod.GET)
    ResponseEntity<List<PastisProfile>> getFiles() throws IOException {

        if (profileService.getFiles() != null) {
            return ResponseEntity.ok(profileService.getFiles());
        }	else {
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

}
